# -*- coding: utf-8 -*-
import gradio as gr


def create_service_manage_page():
    """Service Management"""
    with gr.Column():
        gr.Markdown("# Service Management")
        gr.Markdown("#### Service Management")
