# -*- coding: utf-8 -*-
import gradio as gr


def create_task_manage_page():
    """Task Management"""
    with gr.Column():
        gr.Markdown("# Task Management")
        gr.Markdown("#### Task Management")
